// PAnalysisResultManager.cpp: implementation of the PAnalysisResultManager class.
//
//////////////////////////////////////////////////////////////////////

// #include "stdafx.h"
// #include "../include/AnalysisModuleInterface.h"
// #include "PBuffer.h"
#include "PStringToken.h"

#include "AnalysisResultManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PAnalysisResultManager::PAnalysisResultManager()
{
	m_pAnalysisNotifyInterface = NULL;
	m_pModuleRelayInterface = NULL;
}

// PAnalysisResultManger::PAnalysisResultManager( CAnalysisNotifyInterface* pAnalysisNotifyInterface  );
PAnalysisResultManager::PAnalysisResultManager( CAnalysisNotifyInterface* pAnalysisNotifyInterface , CModuleRelayInterface* pModuleRelayInterface )
{
	m_pAnalysisNotifyInterface = pAnalysisNotifyInterface;
	m_pModuleRelayInterface = pModuleRelayInterface;
}

PAnalysisResultManager::~PAnalysisResultManager()
{
}

CModuleRelayInterface* PAnalysisResultManager::GetModuleRelayInterface()
{
	return m_pModuleRelayInterface;
}	

void PAnalysisResultManager::Initialize( unsigned int uiModuleId , CAnalysisNotifyInterface* pAnalysisNotifyInterface , CModuleRelayInterface* pModuleRelayInterface )
{
	m_uiModuleId = uiModuleId;
	m_pAnalysisNotifyInterface = pAnalysisNotifyInterface;
	m_pModuleRelayInterface = pModuleRelayInterface;
}

void PAnalysisResultManager::AddAnalysisResult( char* pBuffer, int nLength )
{
	if( nLength  > 0 )
	{
		m_Buffer.AddTail( pBuffer, nLength );
	}
}

void PAnalysisResultManager::ProcessBufferData( bool IsEnd )
{
	char* pBuffer 			= m_Buffer.GetBuffer();

	int nOneDataSize		= 0;
	int nSTXIndex			= 0;
	int nETXIndex			= 0;
	int nBufferUsedLength		= m_Buffer.GetBufferUsedLength();

	//  printf( "In ProcessBufferData %s , %d\n" , pBuffer , nBufferUsedLength );
	while( nBufferUsedLength > 0 && -1 != nSTXIndex && -1 != nETXIndex )
	{
		// printf( "m_pBuffer = %s , size = %d\n" , pBuffer  , nBufferUsedLength );
		nSTXIndex = FindAfter( 0 , nBufferUsedLength , pBuffer , STX );
		// printf( "m_pBuffer1 = %s , size = %d\n" , pBuffer  , nBufferUsedLength );
		if( nSTXIndex == 0 )
		{
			nETXIndex = FindAfter( nSTXIndex + strlen( STX ) , nBufferUsedLength , pBuffer , ETX );
			// printf( "m_pBuffer2 = %s , size = %d\n" , pBuffer  , nBufferUsedLength );
			if( nETXIndex > nSTXIndex )
			{
				ProcessOneData( CORRECT_ONE_DATA , pBuffer + nSTXIndex + strlen( STX ) , nETXIndex - strlen( STX ) );
			
				m_Buffer.UsedLength( nETXIndex + strlen( ETX ) );
				// printf( "In pBuffer = %s\n" , pBuffer );
				// printf( "nETXIndex + strlen( ETX ) = %d , UsedLength = %d\n" ,  nETXIndex + strlen( ETX ) , m_Buffer.GetBufferUsedLength() );
			}
			else
			{
				m_Buffer.UsedLength( nSTXIndex );
			}
		}
		else if( nSTXIndex > 0 )
		{
			ProcessOneData( ERROR_ONE_DATA , pBuffer , nSTXIndex );
			m_Buffer.UsedLength( nSTXIndex );
		}
		else if( IsEnd )
		{
			ProcessOneData( ERROR_ONE_DATA , pBuffer , nBufferUsedLength );
			m_Buffer.UsedLength( nBufferUsedLength );
		}

		nBufferUsedLength = m_Buffer.GetBufferUsedLength();
		// printf( "While End pBuffer = %s , UsedLength = %d\n" , m_Buffer.GetBuffer() , nBufferUsedLength );
	}
}

int PAnalysisResultManager::FindAfter( int nAfter , int nLength , char* pBuffer , const char* pFindText )
{
	if( nLength < 2 )
	{
		return -1;
	}

	for( int i = nAfter ; i <= nLength - strlen( ETX )  ; i++ )
	// for( int i = 0 ; i <= nLength - strlen( ETX )  ; i++ )
	{
		if( 0 == strncmp( pBuffer + i , pFindText , strlen( pFindText ) ) )
		{
			return i;
		}
	}

	return -1;
}

void PAnalysisResultManager::ProcessOneData( int nType , char* pBuffer , int nLength )
{
	if( nLength <= 0 )
	{
		return;
	}

	switch( nType )
	{
	case CORRECT_ONE_DATA: 
		{
			const char aSep[] = "|";
			char* szTokenData = NULL;
			
			int nType = 0;

			PStringToken StringToken;

			StringToken.CreateTokenBuf( pBuffer , nLength );
			szTokenData = StringToken.GetFirstToken( aSep );

			nType = atoi( szTokenData );
			// printf( "nType = %d\n", nType );
			if( 0 == nType )
			{
				ProcessAnalysisResult( &StringToken , nLength );
			}
			else if( 1 == nType )
			{
				ProcessAnalysisState( &StringToken , nLength );
			}
			else
			{
			}
			
			StringToken.DeleteTokenBuf();
		}
		break;
	case ERROR_ONE_DATA:
		{
			ProcessAnalysisError( pBuffer , nLength );
		}
		break;
	}
}

void PAnalysisResultManager::ProcessAnalysisResult( PStringToken* pStringToken , int nLength )
{
	int nLineNo = 0;
	int nFileNameSize = 0;
	int nErrorTextSize = 0;
	
	const char aSep[] = "|";
	char* pszTokenData = NULL;

	CAnalysisResultPin AnalysisResultPin;
			
	pszTokenData = pStringToken->GetNextToken( aSep );
	// printf( "Filename = %s\n", szTokenData );
	nFileNameSize = strlen( pszTokenData );
	AnalysisResultPin.m_pszFileName = new char[ nFileNameSize + 1 ];
	memset( AnalysisResultPin.m_pszFileName , 0 , nFileNameSize + 1 );

	strncpy( AnalysisResultPin.m_pszFileName , pszTokenData , nFileNameSize );
	// printf( "In CORRECT_ONE_DATA = %s\n" , AnalysisResultPin.m_pszFileName );
			
	pszTokenData = pStringToken->GetNextToken( aSep );
	// printf( "LineNo = %s\n", szTokenData );
	nLineNo = atoi( pszTokenData );

	AnalysisResultPin.m_nLineNo = nLineNo;
	// printf( "In CORRECT_ONE_DATA = %d\n" , AnalysisResultPin.m_nLineNo );

	pszTokenData = pStringToken->GetNextToken( aSep );
	// printf( "Error Text = %s\n", szTokenData );
	nErrorTextSize = strlen( pszTokenData );
	AnalysisResultPin.m_pszErrorText = new char[ nErrorTextSize + 1 ];
	memset( AnalysisResultPin.m_pszErrorText , 0 , nErrorTextSize + 1 );

	strncpy( AnalysisResultPin.m_pszErrorText , pszTokenData , nErrorTextSize );
	// printf( "In CORRECT_ONE_DATA = %s\n" , AnalysisResultPin.m_pszErrorText );

	if( m_pAnalysisNotifyInterface )
	{	
		m_pAnalysisNotifyInterface->OnNotifyAnalysisResult( m_uiModuleId , &AnalysisResultPin );
	}
}

// void PAnalysisResultManager::ProcessAnalysisState( char* pBuffer , int nLength )
void PAnalysisResultManager::ProcessAnalysisState( PStringToken* pStringToken , int nLength )
{
	int nTextSize = 0;
	int nProcessRate = 0;

	char* pszText = NULL;
	char* pszTokenData = NULL;
	
	const char aSep[] = "|";

	CNotifyProcessState ProcessState;
	ProcessState.m_uiModuleId = m_uiModuleId; 

	pszTokenData = pStringToken->GetNextToken( aSep );
	nProcessRate = atoi( pszTokenData );
	// printf( "Process Rate = %d\n", nProcessRate );

	ProcessState.m_nProcessRate = nProcessRate;

	pszTokenData = pStringToken->GetNextToken( aSep );
	// printf( "Process Text = %s\n", pszTokenData );
	nTextSize = strlen( pszTokenData );
	ProcessState.m_pszText = new char[ nTextSize + 1 ];
	memset( ProcessState.m_pszText , 0 , nTextSize + 1 );

	strncpy( ProcessState.m_pszText , pszTokenData , nTextSize );

	if( m_pAnalysisNotifyInterface )
	{
		m_pAnalysisNotifyInterface->OnNotifyProcessState( &ProcessState );
	}	
}

void PAnalysisResultManager::ProcessAnalysisError( char* pBuffer , int nLength )
{
	char* pData = NULL;

	pData = new char[ nLength + 1 ];
	memset( pData , 0 , nLength + 1 );
	memcpy( pData , pBuffer , nLength );

	// Error No 를 우선 0으로 처리
	m_pAnalysisNotifyInterface->OnNotifyError( m_uiModuleId , 0 , pData );
	
	if( pData )
	{
		delete [] pData;
		pData = NULL;
	}
}
