#include <errno.h>
#include <stdio.h>
#include <stdio_ext.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <sys/wait.h>
#include <termios.h>
#include <stdarg.h>

#include "../include/AnalysisModuleInterface.h"
// #include "AnalysisResultManager.h"
#include "PStringToken.h"
#include "UNOLexer.h"
#include "tree.h"
#include "UNOLexerInfo.h"

// #include "uno.h"

char** g_aArray = NULL;
char g_aOption[ MAX_FILENAME_SIZE ];

extern "C" int UnoCheckEx( int argc , char* argv[] );
// extern "C" int LocalCheck( int argc , char** argv );

extern "C" bool GetModuleInfo( CAnalysisModuleInfo* pInfo )
{ 
	// printf( "In UNOLexer GetModuleInfo\n" );
	pInfo->m_uiId = MODULE_ID;
	memcpy( pInfo->m_szName , MODULE_NAME , strlen( MODULE_NAME ) );
	pInfo->m_uiRevisionNo = MODULE_REVISION_NO; 

	pInfo->m_uiInputId = IOP_SOURCE_FILES;
	memcpy( pInfo->m_szInputName , PIN_NAME_SOURCE_FILE , strlen( PIN_NAME_SOURCE_FILE ) );
	// pInfo->m_szInputDescription  

	pInfo->m_uiOutputId = IOP_PARSING_RESULT;
	memcpy( pInfo->m_szOutputName , PIN_NAME_PARSING_RESULT , strlen( PIN_NAME_PARSING_RESULT ) );
	// pInfo->m_szOutputDescription 

	return true;
}

extern "C" void	Initialize( CAnalysisNotifyInterface* pInterface , CModuleRelayInterface* pRelayInterface )
{
	g_AnalysisResultManager.Initialize( MODULE_ID , pInterface , pRelayInterface );
}

// extern "C" bool StartProcess( CAnalysisNotifyInterface* pInterface , CInOutPinBase* pInputPin , CInOutPinBase** pOutputPin )
extern "C" bool StartProcess( CInOutPinBase* pInputPin )
{
	PStringToken StringToken;
	CSourceFilePin* pSourceFilePin = ( CSourceFilePin*  )pInputPin;
	
	char* pTokenData	= NULL;

	int i			= 0;
	int nCount		= 0;
	int nArgSize		= 0;
	int nSourceFileCount	= pSourceFilePin->m_nFileCount;

	g_ProcessStateInfo.m_nCount =  nSourceFileCount * 2;

	nArgSize = nSourceFileCount;
	g_aArray = ( char** )malloc( sizeof( char* ) * nArgSize );

	for( ; i < nArgSize ; i++ )
	{
		g_aArray[ i ] = ( char* )malloc( MAX_FILENAME_SIZE );
		memset( g_aArray[ i ] , 0 , MAX_FILENAME_SIZE );
	}
	
	StringToken.CreateTokenBuf( pSourceFilePin->m_pszFileName , strlen( pSourceFilePin->m_pszFileName )  );

	pTokenData = StringToken.GetFirstToken( SOURCE_FILENAME_SEPERATOR );
	while( NULL != pTokenData && nCount < nSourceFileCount )
	{
		// printf( "pTokenData = %s , %d\n" , pTokenData ,  strlen( pTokenData ) );
		strncpy( g_aArray[ nCount ] , pTokenData , strlen( pTokenData ) );
		// printf( "nCount = %d, g_aArray[ 0 ] = %s\n" , nCount , g_aArray[ 0 ]  );
		pTokenData = StringToken.GetNextToken( SOURCE_FILENAME_SEPERATOR );
		// printf( "1 - 1 nCount = %d, g_aArray[ 0 ] = %s\n" , nCount , g_aArray[ 0 ]  );
		nCount++;
	}
	StringToken.DeleteTokenBuf(); 

	PrintfEx( "!@1|%d|Analysising... (Target file : %s)#$" , g_ProcessStateInfo.m_nCurrentProcessRate , pSourceFilePin->m_pszFileName );
	// PrintfEx( "!@1|%d|In [Module Name : %s]#$" , g_ProcessStateInfo.m_nCurrentProcessRate , MODULE_NAME );
	UnoCheckEx( nArgSize , g_aArray );

	g_ProcessStateInfo.m_nCurrentProcessRate = 100;
	PrintfEx( "!@1|%d|Termination of analysis\n#$" , g_ProcessStateInfo.m_nCurrentProcessRate );

	FreeArgument( &g_aArray , nArgSize );
	ExitEx( -1 );

	return 0;
}

void NotifyRelayResult( treenode* pParseTree , char* pszFileName )
{
	g_AnalysisResultManager.ProcessBufferData( true );

	if( pParseTree )
	{	
		g_ProcessStateInfo.m_nCurrentProcessRate += ( 90 / g_ProcessStateInfo.m_nCount);
		PrintfEx( "!@1|%d|%s : Sucess of parse-tree creation#$" , g_ProcessStateInfo.m_nCurrentProcessRate  , pszFileName );
		// PrintfEx( "!@1|%d|Start of %s's analysis#$" , g_ProcessStateInfo.m_nCurrentProcessRate , pszFileName );
	
		CParsingResultPin ParsingResultPin;
	
		// ParsingResultPin.m_pOption = g_aOption;
		ParsingResultPin.m_pParseTree = pParseTree;
		// printf( "In NotifyRelayResult ParseTree = %08X\n" , pParseTree );
		strncpy( ParsingResultPin.m_szFileName , pszFileName , strlen( pszFileName ) );
	
		CModuleRelayInterface* pInterface = g_AnalysisResultManager.GetModuleRelayInterface();
		pInterface->OnRelayResult( MODULE_ID , &ParsingResultPin  );
		
		g_ProcessStateInfo.m_nCurrentProcessRate += ( 90 / g_ProcessStateInfo.m_nCount);
		PrintfEx( "!@1|%d|%s : Completion of analysis#$" , g_ProcessStateInfo.m_nCurrentProcessRate , pszFileName );
		// printf( "In NotifyRelayResult End\n" );
	}
	else
	{
		g_ProcessStateInfo.m_nCurrentProcessRate += ( ( 90 / g_ProcessStateInfo.m_nCount) * 2 );
		PrintfEx( "!@2|%s : Failure of parse-tree creation#$" , pszFileName );
	}
}

void PrintfEx( const char* pFmt , ... )
{
	va_list ptr;	

	char* pszBuf	= NULL;

	int nRt 	= 0;
	int nBeforeRt	= MAX_ARG_SIZE;

	va_start( ptr , pFmt );

	pszBuf = ( char* )malloc( nBeforeRt );
	memset( pszBuf , 0 , sizeof( pszBuf ) );

	nRt = vsnprintf( pszBuf , nBeforeRt , pFmt , ptr );
	while( nRt >= nBeforeRt )
	{
		if( pszBuf )
		{
			delete [] pszBuf;
			pszBuf = NULL;
		}

		pszBuf = ( char* )malloc( nRt + 1 );
		memset( pszBuf , 0 , nRt + 1 );

		nBeforeRt = nRt + 1;		
		nRt = vsnprintf( pszBuf , nRt + 1 , pFmt , ptr );
	}
	
	// printf( "%s" , pszBuf );
	// printf( "\nnRt = %d\n" , nRt );

	g_AnalysisResultManager.AddAnalysisResult( pszBuf , nRt );
	g_AnalysisResultManager.ProcessBufferData();
	
	if( pszBuf )
	{
		delete [] pszBuf;
		pszBuf = NULL;
	}

	va_end( ptr );
}

/* void SetArgument( struct Argument* pArgument , unsigned int uiIsLocal )
{
	if( uiIsLocal )
	{

		memcpy( &g_LocalArgument , pArgument , sizeof( CArgument ) );
		printf( "In UNOLexer.cpp address = %08X , Count = %d \n" , g_LocalArgument.m_ppArgument , g_LocalArgument.m_nArgumentCount );
	}
	else
	{
		memcpy( &g_GlobalArgument , pArgument , sizeof( CArgument ) );
	}
} */

void ExitEx( int nExitNo )
{
	/*
 * 	FreeArgument( &g_MainArgument );
	FreeArgument( &g_LocalArgument ); 
	FreeArgument( &g_GlobalArgument ); 
	*/

	g_AnalysisResultManager.ProcessBufferData( true );

	if( -1 != nExitNo )
	{
		exit( nExitNo );
	}
}

/* void  FreeArgument( CArgument* pArgument )
{
	printf( "Before FreeArgument %08X \n" , pArgument->m_ppArgument );
	FreeArgument( pArgument->m_ppArgument , pArgument->m_nArgumentCount );
	printf( "After FreeArgument %08X \n" , pArgument->m_ppArgument );
} */

void FreeArgument( char*** pppArgument , int nArgSize )
{
	for( int i = 0; i < nArgSize ; i++ )
	{
		if( ( *pppArgument )[ i ] )
		{
			free( ( *pppArgument )[ i ] );
			( *pppArgument )[ i ] = NULL;
		}
	}

	if( *pppArgument )
	{
		free( *pppArgument );
		*pppArgument = NULL;
	}
}

/* void FreeArgument( char** ppArgument , int nArgSize )
{
	for( int i = 0; i < nArgSize ; i++ )
	{
		if( ppArgument[ i ] )
		{
			free( ppArgument[ i ] );
			ppArgument[ i ] = NULL;
		}
	}

	if( ppArgument )
	{
		free( ppArgument );
		ppArgument = NULL;
	}
} */
